package de.fraunhofer.iem.dataprovider.kpi.enumeration

import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeChildResponseDto
import de.fraunhofer.iem.dataprovider.kpi.dto.KPITreeResponseDto

enum class KpiKind {
    // Raw Value KPIs
    CHECKED_IN_BINARIES {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Checked in Binary",
                description = "Used to assess the compliance to the OpenCoDE " +
                        "platform guidelines in regards of not checking in binaries.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    NUMBER_OF_COMMITS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Number of Commits",
                description = "Total number of commits on the default branch of the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    VULNERABILITY_SCORE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            // TODO: in order to provide more detailed information here we need to query the ORT api
            return KPITreeResponseDto(
                value = value,
                name = "Vulnerability Score",
                description = "A vulnerability with this score was " +
                        "found in the projects dependencies. Further information are not disclosed here.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    NUMBER_OF_SIGNED_COMMITS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Number of Signed Commits",
                description = "Total number of signed and verified commits on the default branch of the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    IS_DEFAULT_BRANCH_PROTECTED {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Default Branch Protection",
                description = "Used to assess compliance with a standard development process." +
                        " For this purpose, it is examined whether the standard development" +
                        " branch is protected against unintentional changes.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    SECRETS {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Secrets",
                description = "Used to look for suspicious strings in the repository," +
                        " which might indicate leaked passwords or other secrets.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    SAST_USAGE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "SAST Usage",
                description = "Used to approximate the usage of SAST tools during development.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },

    // Calculated KPIs
    SIGNED_COMMITS_RATIO {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Commit Signature Ratio",
                description = "Used to assess compliance with a common and " +
                        "transparent development process. It is desirable that all commits " +
                        "are signed by their authors. Therefore, the ratio of signed commits " +
                        "to all commits is determined to calculate this metric.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    PROCESS_COMPLIANCE {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Process Compliance Score",
                description = "Assesses the development process of the software provided." +
                        " For this purpose, the development process traceable in the repository is" +
                        " compared with common development standards to enable an assessment.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    PROCESS_TRANSPARENCY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Process Transparency Score",
                description = "Assesses the transparency resp. traceability of the development " +
                        "process of the provided software for external parties. For this purpose," +
                        " various analyzes are performed that assess the availability of information " +
                        "about the software development process within the repository.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    SECURITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Security Score",
                description = "Assesses the security of the software provided. For this purpose, " +
                        "various security-relevant analyzes are carried out, which, among other things," +
                        " check the external dependencies or the code for vulnerabilities.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },
    MAXIMAL_VULNERABILITY {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Maximal Dependency Vulnerability Score",
                description = "This score is calculated by the following formula: 100 - (max(CVSS score) * 10). " +
                        "Thus, a lower value indicates a more critical vulnerability.",
                children = children,
                isEmpty = isEmpty
            )
        }
    },

    // ROOT
    ROOT {
        override fun toViewModel(
            value: Int,
            children: List<KPITreeChildResponseDto>,
            isEmpty: Boolean
        ): KPITreeResponseDto {
            return KPITreeResponseDto(
                value = value,
                name = "Project Score",
                description = "Assesses the project resp. the provided software in the aspects of" +
                        " maturity (based on quality, security and usability aspects) as well as development process.",
                isRoot = true,
                children = children,
                isEmpty = isEmpty
            )
        }
    };

    abstract fun toViewModel(
        value: Int,
        children: List<KPITreeChildResponseDto>,
        isEmpty: Boolean
    ): KPITreeResponseDto
}
